package compbio.controllers;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import compbio.cassandra.DataBase;
import compbio.cassandra.ProteinBean;
import compbio.cassandra.Total;
import compbio.cassandra.TotalByCounterBean;
import compbio.statistic.CassandraRequester;

/**
 * Spring controller for sequence queries. This version works in the servlet
 * style.
 * 
 * @author Alexander Sherstnev
 * @author Natasha Sherstneva
 * 
 * @since 0.5
 * @version 1.0 December 2013
 */
@Controller
@RequestMapping("/sequence")
public class SequenceController extends BasicController {

	/**
	 * pattern for NON-protein alphabet symbols
	 */
	private final Pattern NONPROTEIN = Pattern.compile("[^ARNDCQEGHILKMFPSTWYV]+", Pattern.CASE_INSENSITIVE);

	@RequestMapping(value = "query", method = RequestMethod.GET)
	public String formSequenceQuery(Map<String, Object> model) {
		model.put("username", getPrincipalName());
		model.put("value", "AAAAA");
		return "query/Sequence";
	}

	@RequestMapping(value = "counts/query", method = RequestMethod.GET)
	public String formCounterQuery(Map<String, Object> model) {
		model.put("username", getPrincipalName());
		model.put("value", 5);
		return "query/SequenceCounts";
	}

	@RequestMapping(value = "sequence/results", method = RequestMethod.GET)
	public String findSequence(@RequestParam("sequence") String sequence, @RequestParam("searchtype") String searchtype,
			Map<String, Object> model) {
		model.put("username", getPrincipalName());
		final long startTime = System.currentTimeMillis();

		// input checks
		String trimmedsequence = sequence.replaceAll("\\s", "");
		if (trimmedsequence.equalsIgnoreCase("")) {
			model.put("error", "The sequence cann't be empty");
			model.put("value", sequence);
			return "query/Sequence";
		}
		if (NONPROTEIN.matcher(trimmedsequence).find()) {
			model.put("error", "The sequence contains symbols not from the standard protein alphabet");
			model.put("value", sequence);
			return "query/Sequence";
		}

		model.put("njobs", 0);
		model.put("prot", trimmedsequence);
		model.put("searchtype", searchtype);
		String csvline = "";
		if (0 < trimmedsequence.length()) {
			CassandraRequester cr = new CassandraRequester();
			List<ProteinBean> r = cr.readProteins(trimmedsequence, searchtype);
			model.put("results", r);
			if (null != r) {
				if (searchtype.equals("whole"))
					model.put("njobs", r.get(0).getJobid().size());
				else
					model.put("njobs", r.size());
				csvline = "\'Job\',\'Annotation\',\'Sequence\'%0A";
			}
			// form CSV file string
			for (ProteinBean entry : r) {
				List<String> jobs = entry.getJobid();
				String protein = entry.getSequence();
				LinkedHashMap<String, String> predictions = entry.getPredictions();
				for (String job : jobs) {
					csvline += "\'" + job + "\',\'Sequence\',\'" + protein + "\',\'%0A";
					for (Map.Entry<String, String> pr : predictions.entrySet()) {
						csvline += "\'\',\'" + pr.getKey() + "\',\'" + pr.getValue() + "\'%0A";
					}
				}
			}
		}
		model.put("csvfile", csvline);

		final long endTime = System.currentTimeMillis();
		model.put("timeExecution", (endTime - startTime));
		return "reportProteinSequences";
	}

	@RequestMapping(value = "counts/results", method = RequestMethod.GET)
	public String countSequences(@RequestParam("counterJob") String counter, Map<String, Object> model) {
		model.put("username", getPrincipalName());
		final long startTime = System.currentTimeMillis();

		if (counter.equals("")) {
			model.put("error", "The value must not be empty");
			model.put("value", counter);
			return "query/SequenceCounts";
		}

		int realcounter;
		try {
			realcounter = Integer.parseInt(counter.trim());
		} catch (NumberFormatException e) {
			model.put("error", "The value must be an integer number");
			model.put("value", counter);
			return "query/SequenceCounts";
		}

		if (realcounter < 1) {
			model.put("error", "The value must be greater than 0");
			model.put("value", counter);
			return "query/SequenceCounts";
		}

		CassandraRequester cr = new CassandraRequester();
		List<TotalByCounterBean> r = cr.readProteinByCounter(realcounter);
		model.put("njobs", 0);
		String csvline = "";
		if (null != r) {
			model.put("njobs", r.size());
			csvline = "\'Job%20 count\', \'Protein%20Sequence\'%0A";
		}
		// form line for CSV file

		for (TotalByCounterBean b : r) {
			if (b.getName().equals("")) {
				csvline += "\'" + b.getTotaljobs() + "\',\'Alignment%20job\'%0A";
				// fix problem with records without protein sequence (alignment
				// jobs)
				b.setName("Alignment job");
			} else {
				csvline += "\'" + b.getTotaljobs() + "\',\'" + b.getName() + "\'%0A";
			}
		}
		model.put("csvfile", csvline);

		model.put("results", r);
		final long endTime = System.currentTimeMillis();
		model.put("timeExecution", (endTime - startTime));
		model.put("counter", realcounter);
		return "reportProteinSequencesCounter";
	}

}
